// NewsWatcher 2.1 "myTools.h"
// Copyright (C) 1994-1996 Mizutori Tetsuya
// 1994/5/24, 1994/7/15, 1995/1/15, 1995/2/8, 1995/7/14, 1996/8/27, 1996/11/24

// Define MZTR patch levels
#define MZTR_PATCH			1

#include "myMime.h"

#define MIMEMARK	'='
#define MIMESEP		'?'

// Define Constants and Macros
// JIS Kanji sequence
// New JIS: ESC + '$' + 'B' + [ JIS codes ] + ESC + '(' + 'J'
// Old JIS: ESC + '$' + '@' + [ JIS codes ] + ESC + '(' + 'B'

#define	ESC		'\033'
#define	KIN		'$'
#define	KIN1	'B'
#define	KIN2	'@'
#define	KOUT	'('
#define	KOUT1	'J'
#define	KOUT2	'B'

// Global and Constant variables


// Prototype

/***** Keyboard sensing *****/
//Boolean	IsPressed ( unsigned short k );

/**** Kanji code conversion *****/
Boolean IsJISKanji	( const char * p, const char * pEnd );
Boolean IsMSKanji	( const char * p, const char * pEnd );
Boolean IsEUCKanji  ( const char * p, const char * pEnd );

void EUCKanji2JIS	( const char * p, char * q );
void JIS2EUCKanji	( const char * p, char * q );
void MSKanji2JIS	( const char * p, char * q );
void JIS2MSKanji	( const char * p, char * q );


Boolean IsKanji		( const char * p, const char * pStart, const char * pEnd );
long SkipOnBrace	( const char * p, const char * pStart, const char * pEnd );

OSErr MyResizeHandle	( void * handle, Size len );

long GetFillColumn	( void );

Boolean KillCustomMessageID	( void );


// Define macros
// Macros used in 'net.c'

#define if_COUNT_MS_KANJI(n,p,pEnd)	\
	if ( IsMSKanji( p, pEnd ) )									\
	{															\
		n += 3;													\
		while ( p < pEnd && IsMSKanji( p, pEnd ) )  p += 2;		\
		n += 3;													\
	}


#define if_MS_KANJI(p,q,pEnd)	\
	if ( IsMSKanji( p, pEnd ) )									\
	{															\
		*q++ = ESC; *q++ = KIN; *q++ = KIN1;					\
		while ( p < pEnd && IsMSKanji( p, pEnd ) ) {			\
			MSKanji2JIS( p, q );								\
			p += 2; q += 2; }									\
		*q++ = ESC; *q++ = KOUT; *q++ = KOUT1;					\
	}


#define if_JIS_KANJI(p,q,pEnd)	\
	if ( *p==ESC && *(p+1)==KIN && ( *(p+2)==KIN1 || *(p+2)==KIN2) )	\
	{															\
		p += 3;													\
		while ( p < pEnd && (*p != CR || *(p+1) != LF) ) {		\
			if ( *p==ESC && *(p+1)==KOUT && ( *(p+2)==KOUT1 || *(p+2)==KOUT2) ) {	\
				p += 3; break; }								\
			if ( ! IsJISKanji( p, pEnd ) ) break;				\
			JIS2MSKanji( p, q );								\
			p += 2; q += 2;										\
		}														\
	}


#define if_JIS_MIME(p,q,pEnd)	\
	if ( *p==MIMEMARK && *(p+1)==MIMESEP )	\
	{															\
		unsigned long	pLen;									\
		unsigned long	qLen;									\
		pLen = MimeDecodeText((unsigned char *)p,(pEnd-p),(unsigned char *)q,&qLen);	\
		if ( pLen > 0 ) {										\
			char *		s = q;									\
			char *		sEnd = s+qLen;							\
			while ( s < sEnd ) {								\
				if_JIS_KANJI(s,q,sEnd)							\
				else											\
					*q++ = *s++;								\
			}													\
			p += pLen;											\
		} else {												\
			*q++ = *p++;										\
			*q++ = *p++;										\
		}														\
	}


// Macros used in 'news.c'

#define if_JIS_KANJI_CHUNK(p,t,tLen,c,gFileLinePos)	\
	if (c==ESC && d==KIN) {				\
		gEscapeKanjiIn = true;				\
		t++; c = *t; ++t;					\
		tLen -= 2;							\
	} else if ( gEscapeKanjiIn ) {		\
		if (c==KIN1 || c==KIN2) {			\
			gInKanjiSequence = true;		\
		} else {							\
			*p++ = ESC;						\
			*p++ = KIN;						\
			*p++ = c;						\
		}									\
		gEscapeKanjiIn = false;				\
		c = d;								\
		t++;								\
		tLen--;								\
	} else if (c==ESC && d==KOUT) {		\
		gEscapeKanjiOut = true;				\
		t++; c = *t; ++t;					\
		tLen -= 2;							\
	} else if ( gEscapeKanjiOut ) {		\
		if (c==KOUT1 || c==KOUT2) {			\
			gInKanjiSequence = false;		\
		} else {							\
			*p++ = ESC;						\
			*p++ = KOUT;					\
			*p++ = c;						\
		}									\
		gEscapeKanjiOut = false;			\
		c = d;								\
		t++;								\
		tLen--;								\
	} else if ( gInKanjiSequence ) {	\
		char	str[2];						\
		str[0] = c; str[1] = d;				\
		if ( IsJISKanji( str, str+2 ) ) {	\
			JIS2MSKanji( str, p );			\
			p += 2;							\
			t += 2;	tLen -= 2;				\
			gFileLinePos += 2;				\
			c = *(t-1);						\
		} else {							\
			gInKanjiSequence = false;		\
		}									\
	}


#define if_JIS_MIME_CHUNK(p,t,tLen,c,gFileLinePos)	\
	if (c==MIMEMARK && d==MIMESEP && gMimeStart==nil) {		\
		gMimeStart = p;						\
		gFileLinePos += 1;					\
		*p++ = c;							\
		c = d; 								\
		t++;								\
		tLen--;								\
	} else if (c==MIMESEP && d==MIMEMARK && gMimeStart!=nil) {	\
		Ptr				q = gMimeStart;		\
		Ptr				pEnd;				\
		unsigned long	pLen;				\
		unsigned long	qLen;				\
		*p++ = c;							\
		*p++ = d;							\
		pLen = MimeDecodeText((unsigned char *)q,(p-q),(unsigned char *)q,&qLen);	\
		if ( pLen > 0 ) {					\
			p = gMimeStart;					\
			q = p;							\
			pEnd = p + qLen;				\
			while ( p < pEnd ) 				\
				if_JIS_KANJI(p,q,pEnd)		\
				else p++;					\
			p = q;							\
		}									\
		t++; c = *t; ++t;					\
		tLen -= 2;							\
		gFileLinePos += 2 - (gMimeStart + pLen - p);	\
		gMimeStart = nil;					\
	}

// end of header file
